@@ -43,11 +43,18 @@ has use_microsoft_dot_net_compatible_separator => (
 # By default the SOAP call uses positional parameters. If this flag is set,
 # the argument list to the call is interpreted as a Hash
 has named_parameters => (
-    is => 'rw',    
+    is => 'rw',
     isa => 'ArrayRef|Str|Undef',
-    trigger => \&_convert_parameters,    
+    trigger => \&_convert_parameters,
     );
 
+has attrmap => (
+    is  => 'rw',
+    isa => 'HashRef',
+    required => 0,
+    predicate => 'has_attrmap'
+);
+
 has use_net_ssl => (
     is => 'rw',
     isa => 'Bool',
@@ -84,16 +91,16 @@ has ca_certificate_path => (
     is => 'rw',
     isa => 'Str',
     );
-    
+
 has ca_certificate_file => (
     is => 'rw',
     isa => 'Str',
-    );    
+    );
 
 sub BUILD {
     my $self = shift;
     if ($self->do_not_use_charset) {
-	$SOAP::Constants::DO_NOT_USE_CHARSET = 1;
+    $SOAP::Constants::DO_NOT_USE_CHARSET = 1;
     }
 }
 
@@ -111,11 +118,35 @@ sub _convert_parameters {
 
 }
 
-
 sub _build_config {
     my $self = shift;
 }
 
+# override this to add authentication headers
+# must return an array (not array ref!) with SOAP::Head objects
+sub _make_head {
+    return;
+}
+
+# can be overriden in case you need a more complex data structure
+sub _make_params {
+    my $self = shift;
+    my $args = shift;
+
+    my @params;
+    if (ref $self->named_parameters eq 'ARRAY' && scalar @{$self->named_parameters}) {
+        my @args = @{$args};
+        foreach my $key (@{$self->named_parameters}) {
+            my $value = shift @args;
+            push @params, SOAP::Data->new(name => $key, value => $value );
+            $self->log()->debug('Named parameter: ' . $key . ' => ' . $value );
+        }
+    } else {
+        @params = @{$args};
+        $self->log()->debug('Parameters: ' . join(', ', @params));
+    }
+    return @params;
+}
 
 sub _soap_call {
     my $self = shift;
@@ -128,125 +159,115 @@ sub _soap_call {
 
     my $client = SOAP::Lite
         ->uri($self->uri);
-        
+
     $self->log()->debug('Performing SOAP call to method ' . $self->method . ' on service ' . $self->uri . ' via ' . $proxy);
-        
-    # Force usage of net::ssl 
+
+    # Force usage of net::ssl
     if ($self->use_net_ssl()) {
-        use Net::SSL;
+        require Net::SSL;
         $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "Net::SSL";
-    
+
         if ($self->certificate_p12_file) {
             $ENV{HTTPS_PKCS12_FILE}  = $self->certificate_p12_file;
-            
+
             if ($self->certificate_key_file || $self->certificate_file) {
                 die "Options certificate_file/certificate_key_file and certificate_p12_file are mutually exclusive";
             }
-            
+
             if ($self->certificate_key_password) {
                 $ENV{HTTPS_PKCS12_PASSWORD}  = $self->certificate_key_password;
             }
         }
-    
+
         if ($self->certificate_key_file) {
             if ($self->certificate_key_password) {
-                die "Net::SSL does not support password protected keys - use certificate_p12_file instead";     
+                die "Net::SSL does not support password protected keys - use certificate_p12_file instead";
             }
-                        
+
             if (!$self->certificate_file) {
                 die "You need to pass certificate AND key file, use certificate_p12_file to pass a PKCS12";
             }
-            
+
             $ENV{HTTPS_KEY_FILE}  = $self->certificate_key_file;
-            $ENV{HTTPS_CERT_FILE} = $self->certificate_file;                 
-            
+            $ENV{HTTPS_CERT_FILE} = $self->certificate_file;
+
         } elsif ($self->certificate_file) {
             die "You need to pass certificate AND key file, use certificate_p12_file to pass a PKCS12";
         }
-        
+
         if ($self->ca_certificate_path) {
             $ENV{HTTPS_CA_DIR} = $self->ca_certificate_path;
         }
-        
+
         if ($self->ca_certificate_file) {
             $ENV{HTTPS_CA_FILE} = $self->ca_certificate_file;
         }
-                
-        $client->proxy($proxy); 
-        
+
+        $client->proxy($proxy);
+
         $self->log()->trace('Using Net::SSL, EVN is' . Dumper $ENV);
-        
+
     } # end of Net:SSL, IO::Socket::SSL
     elsif( $proxy =~ /^https:/i ) {
-                
+
         use IO::Socket::SSL;
         $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "IO::Socket::SSL";
-        
-        my $ssl_opts = { 
+
+        my $ssl_opts = {
             verify_hostname => ($self->ssl_ignore_hostname ? 0 : 1)
         };
-        
+
         if ($self->certificate_p12_file) {
             die "Using pkcs12 containers is not supported by IO::Socket::SSL"
         }
-        
+
         if ($self->certificate_key_file) {
             if (!$self->certificate_file) {
                 die "You need to pass certificate AND key file";
             }
-            $ssl_opts->{SSL_key_file}  = $self->certificate_key_file;            
-            $ssl_opts->{SSL_cert_file}  = $self->certificate_file;    
-            
+            $ssl_opts->{SSL_key_file}  = $self->certificate_key_file;
+            $ssl_opts->{SSL_cert_file}  = $self->certificate_file;
+
             if ( $self->certificate_key_password ) {
                 $ssl_opts->{SSL_passwd_cb} = sub { return $self->certificate_key_password; };
             }
-            
+
         } elsif ($self->certificate_file) {
             die "You need to pass certificate AND key file";
         }
-                  
-        if ($self->ca_certificate_path) {	   
-	        $ssl_opts->{SSL_ca_path}  = $self->ca_certificate_path;
+
+        if ($self->ca_certificate_path) {
+            $ssl_opts->{SSL_ca_path}  = $self->ca_certificate_path;
         }
-        
-        if ($self->ca_certificate_file) {      
+
+        if ($self->ca_certificate_file) {
             $ssl_opts->{SSL_ca_file}  = $self->ca_certificate_file;
         }
-        # For whatever reason LWP wants the ssl_opts as ARRAYref!    
+        # For whatever reason LWP wants the ssl_opts as ARRAYref!
         $client-> proxy($proxy, ssl_opts => [%{$ssl_opts}] );
-                
+
         $self->log()->trace('Using IO::Socket::SSL with options ' . Dumper $ssl_opts);
     } else {
         # No ssl
-        $client->proxy($proxy);         
+        $client->proxy($proxy);
     }
-    
+
     if ($self->use_microsoft_dot_net_compatible_separator) {
         # This modifies the seperator used in the SOAPAction Header to add
-        # the method to the uri. Default is # but .Net expects a slash.  
-	    $client->on_action( sub { join('/', @_) } );
-    }
-
-    my @params;
-    if (ref $self->named_parameters eq 'ARRAY' && scalar @{$self->named_parameters}) {
- 	    foreach my $key (@{$self->named_parameters}) {
- 	        my $value = shift @args;
- 	        push @params, SOAP::Data->new(name => $key, value => $value );
-	        $self->log()->debug('Named parameter: ' . $key . ' => ' . $value );
- 	    }
-    } else {
-	   @params = @args;
-	   $self->log()->debug('Parameters: ' . join(', ', @params));
+        # the method to the uri. Default is # but .Net expects a slash.
+        $client->on_action( sub { join('/', @_) } );
     }
 
+    my @params = $self->_make_head();
+    push @params, $self->_make_params( \@args );
 
     my $som;
     eval {
         $som = $client->call($self->method, @params);
     };
     if ($@) {
-	   $self->log()->error('SOAP call died: ' . $@);
-	   die 'Fatal SOAP Error: ' . $@ . " [method=" . $self->method . ", params=(" . join(', ', @params) . ")]";
+       $self->log()->error('SOAP call died: ' . $@);
+       die 'Fatal SOAP Error: ' . $@ . " [method=" . $self->method . ", params=(" . join(', ', @params) . ")]";
     }
 
 
@@ -254,8 +275,8 @@ sub _soap_call {
     %ENV = %ENV_BACKUP;
 
     if ($som->fault) {
-	   $self->log()->error('SOAP call returned error: ' . $som->fault->{faultstring});
-	   die $som->fault->{faultstring};
+       $self->log()->error('SOAP call returned error: ' . $som->fault->{faultstring});
+       die $som->fault->{faultstring};
     }
 
     return $som->result;
@@ -268,9 +289,22 @@ sub get {
     my $result = $self->_soap_call(@_);
     return if (! defined $result);
 
-    if (ref $result ne '') {
-	   die "SOAP call result is not a scalar";
+    if ((ref $result eq 'HASH') && $self->has_attrmap()) {
+        my @keys = keys %{$self->attrmap()};
+        if (scalar @keys != 1) {
+            $self->log()->error('SOAP result is hash but attrmap has more than one item');
+            die 'SOAP result is hash but attrmap has more than one item';
+        }
+        if (!defined $result->{$keys[0]}) {
+            return $self->_node_not_exists();
+        }
+        return $result->{$keys[0]};
     }
+
+    if (ref $result ne '') {
+       die "SOAP call result is not a scalar";
+    }
+
     return $result;
 }
 
@@ -286,7 +320,7 @@ sub get_list {
 
     my $result = $self->_soap_call(@_);
 
-    return if (! defined $result);
+    return $self->_node_not_exists() if (! defined $result);
 
     if (ref $result ne 'ARRAY' ) {
         die "SOAP call result is not a list";
@@ -307,12 +341,22 @@ sub get_hash {
 
     my $result = $self->_soap_call(@_);
 
-    return if (! defined $result);
+    return $self->_node_not_exists() if (! defined $result);
 
     if (ref $result ne 'HASH' ) {
         die "SOAP call result is not a hash";
     }
 
+
+    my $res;
+    if ($self->has_attrmap()) {
+        my %map = %{$self->attrmap()};
+        foreach my $key (keys %map) {
+            $res->{ $map{$key} } = $result->{$key};
+        }
+        return $res;
+    }
+
     return $result;
 }
 
@@ -342,9 +386,9 @@ Connector::Proxy::SOAP::Lite
 
 =head 1 DESCRIPTION
 
-Make a SOAP call using the SOAP::Lite package. Use get if your SOAP call 
-expects a scalar result or get_hash for a hashref. get_meta, get_list and 
-set methods are not supported yet.    
+Make a SOAP call using the SOAP::Lite package. Use get if your SOAP call
+expects a scalar result or get_hash for a hashref. get_meta, get_list and
+set methods are not supported yet.
 
 =head1 USAGE
 
@@ -358,9 +402,9 @@ set methods are not supported yet.
 
 =head2 additional options
 
-=over 
+=over
 
-=item do_not_use_charset 
+=item do_not_use_charset
 
 Boolean, sets $SOAP::Constants::DO_NOT_USE_CHARSET = 1;
 
@@ -368,24 +412,32 @@ Boolean, sets $SOAP::Constants::DO_NOT_USE_CHARSET = 1;
 
 Boolean, set the parameter seperator to "/" (forward slash)
 
-=item named_parameters 
+=item named_parameters
 
 By default, the passed arguments are used as postional arguments in the
 soap call. If you want to use a named parameter, set this to a list of names
 used as keys with the passed parameters. If you pass a string, it is split
 into a list a the whitespace character (usefull with Config::Std, etc).
 
+=item attrmap
+
+Optional, if set keys of the returned hash are mapped from the given hash.
+Keys must be the names of the SOAP response fields, values are the names of
+the keys in the connector response. Can be used with I<get> to extract a
+single field from the response, must contain one element, value is ignored.
+
+
 =back
 
 =head2 SSL support
 
-This connector supports client authentication using certificates.  
+This connector supports client authentication using certificates.
 
-=over 
+=over
 
-=item use_net_ssl 
+=item use_net_ssl
 
-Set this to a true value to use Net::SSL as backend library (otherwise 
+Set this to a true value to use Net::SSL as backend library (otherwise
 IO::Socket::SSL is used). Be aware the Net::SSL does not check the hostname
 of the server certificate so Man-in-the-Middle-Attacks might be possible.
 You should use this only with a really good reason or if you need support
@@ -393,8 +445,8 @@ for PKCS12 containers.
 
 =item ssl_ignore_hostname
 
-Do not validate the hostname of the server certificate (only useful with 
-IO::Socket::SSL as Net::SSL does not check the hostname at all). 
+Do not validate the hostname of the server certificate (only useful with
+IO::Socket::SSL as Net::SSL does not check the hostname at all).
 
 =item certificate_file
 
@@ -404,14 +456,14 @@ Path to a PEM encoded certificate file.
 
 Path to a PEM encoded key file.
 
-=item certificate_p12_file 
+=item certificate_p12_file
 
-Path to a PKCS12 container file. This is only supported by Net:SSL and can 
-not be used together with certificate_file/certificate_key_file. 
+Path to a PKCS12 container file. This is only supported by Net:SSL and can
+not be used together with certificate_file/certificate_key_file.
 
 =item certificate_key_password
 
-The plain password of your encrypted key or PKCS12 container. Note that 
+The plain password of your encrypted key or PKCS12 container. Note that
 Net::SSL does not support password protected keys. You need to use a PKCS12
 container instead! Leave this empty if your key is not protected by a password.
 
